@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2013-2015, Intel Corporation                                **
+** Copyright (c) 2013-2016, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -31,31 +31,44 @@
 #ifndef LIBXSMM_MACROS_H
 #define LIBXSMM_MACROS_H
 
-#define LIBXSMM_STRINGIFY(SYMBOL) #SYMBOL
+#define LIBXSMM_STRINGIFY2(SYMBOL) #SYMBOL
+#define LIBXSMM_STRINGIFY(SYMBOL) LIBXSMM_STRINGIFY2(SYMBOL)
 #define LIBXSMM_TOSTRING(SYMBOL) LIBXSMM_STRINGIFY(SYMBOL)
 #define LIBXSMM_CONCATENATE2(A, B) A##B
 #define LIBXSMM_CONCATENATE(A, B) LIBXSMM_CONCATENATE2(A, B)
 #define LIBXSMM_FSYMBOL(SYMBOL) LIBXSMM_CONCATENATE2(SYMBOL, _)
 #define LIBXSMM_UNIQUE(NAME) LIBXSMM_CONCATENATE(NAME, __LINE__)
 
+#define LIBXSMM_VERSION3(MAJOR, MINOR, UPDATE) (MAJOR * 10000 + MINOR * 100 + UPDATE)
+#define LIBXSMM_VERSION4(MAJOR, MINOR, UPDATE, PATCH) (MAJOR * 100000000 + MINOR * 1000000 + UPDATE * 10000 + PATCH)
+
 #if defined(__cplusplus)
-# define LIBXSMM_EXTERN_C extern "C"
-# define LIBXSMM_INLINE inline
 # define LIBXSMM_VARIADIC ...
+# define LIBXSMM_EXTERN_C extern "C"
+# define LIBXSMM_INLINE_KEYWORD inline
+# define LIBXSMM_INLINE LIBXSMM_INLINE_KEYWORD
 #else
-# define LIBXSMM_EXTERN_C
 # define LIBXSMM_VARIADIC
-# if defined(__STDC_VERSION__) && (199901L <= (__STDC_VERSION__))
+# define LIBXSMM_EXTERN_C extern
+# if defined(__STDC_VERSION__) && (199901L <= (__STDC_VERSION__)) /*C99*/
 #   define LIBXSMM_PRAGMA(DIRECTIVE) _Pragma(LIBXSMM_STRINGIFY(DIRECTIVE))
 #   define LIBXSMM_RESTRICT restrict
-#   define LIBXSMM_INLINE static inline
+#   define LIBXSMM_INLINE_KEYWORD inline
 # elif defined(_MSC_VER)
-#   define LIBXSMM_INLINE static __inline
-# else
-#   define LIBXSMM_INLINE static
-# endif /*C99*/
+#   define LIBXSMM_INLINE_KEYWORD __inline
+# endif
+# if !defined(LIBXSMM_INLINE_KEYWORD)
+#   define LIBXSMM_INLINE_KEYWORD
+# endif
+# define LIBXSMM_INLINE static LIBXSMM_INLINE_KEYWORD
 #endif /*__cplusplus*/
 
+#if defined(LIBXSMM_BUILD)
+# define LIBXSMM_INLINE_EXPORT LIBXSMM_EXTERN_C
+#else
+# define LIBXSMM_INLINE_EXPORT LIBXSMM_INLINE
+#endif
+
 #if !defined(LIBXSMM_RESTRICT)
 # if ((defined(__GNUC__) && !defined(__CYGWIN32__)) || defined(__INTEL_COMPILER)) && !defined(_WIN32)
 #   define LIBXSMM_RESTRICT __restrict__
@@ -76,7 +89,11 @@
 
 #if defined(_MSC_VER)
 # define LIBXSMM_MESSAGE(MSG) LIBXSMM_PRAGMA(message(MSG))
-#elif (40400 <= (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__))
+#elif (LIBXSMM_VERSION3(4, 4, 0) <= LIBXSMM_VERSION3(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__))
+# define LIBXSMM_MESSAGE(MSG) LIBXSMM_PRAGMA(message MSG)
+#elif (LIBXSMM_VERSION3(3, 5, 0) <= LIBXSMM_VERSION3(__clang_major__, __clang_minor__, __clang_patchlevel__)) \
+  && 0 /* message format looks awful; disabled */
+#   define LIBXSMM_PRAGMA(DIRECTIVE) _Pragma(LIBXSMM_STRINGIFY(DIRECTIVE))
 # define LIBXSMM_MESSAGE(MSG) LIBXSMM_PRAGMA(message MSG)
 #else
 # define LIBXSMM_MESSAGE(MSG)
@@ -103,7 +120,7 @@
 #endif
 
 #if defined(__INTEL_COMPILER)
-# define LIBXSMM_PRAGMA_FORCEINLINE LIBXSMM_PRAGMA(forceinline recursive)
+# define LIBXSMM_PRAGMA_FORCEINLINE LIBXSMM_PRAGMA(forceinline)
 # define LIBXSMM_PRAGMA_LOOP_COUNT(MIN, MAX, AVG) LIBXSMM_PRAGMA(loop_count min(MIN) max(MAX) avg(AVG))
 # define LIBXSMM_PRAGMA_UNROLL_N(N) LIBXSMM_PRAGMA(unroll(N))
 # define LIBXSMM_PRAGMA_UNROLL LIBXSMM_PRAGMA(unroll)
@@ -115,20 +132,6 @@
 # define LIBXSMM_PRAGMA_UNROLL
 #endif
 
-#if !defined(LIBXSMM_UNUSED)
-# if 0 /*defined(__GNUC__) && !defined(__clang__) && !defined(__INTEL_COMPILER)*/
-#   define LIBXSMM_UNUSED(VARIABLE) LIBXSMM_PRAGMA(unused(VARIABLE))
-# else
-#   define LIBXSMM_UNUSED(VARIABLE) (void)(VARIABLE)
-# endif
-#endif
-
-#if defined(__GNUC__) || (defined(__INTEL_COMPILER) && !defined(_WIN32))
-# define LIBXSMM_UNUSED_ARG LIBXSMM_ATTRIBUTE(unused)
-#else
-# define LIBXSMM_UNUSED_ARG
-#endif
-
 /*Based on Stackoverflow's NBITSx macro.*/
 #define LIBXSMM_NBITS02(N) (0 != ((N) & 2/*0b10*/) ? 1 : 0)
 #define LIBXSMM_NBITS04(N) (0 != ((N) & 0xC/*0b1100*/) ? (2 + LIBXSMM_NBITS02((N) >> 2)) : LIBXSMM_NBITS02(N))
@@ -136,8 +139,9 @@
 #define LIBXSMM_NBITS16(N) (0 != ((N) & 0xFF00) ? (8 + LIBXSMM_NBITS08((N) >> 8)) : LIBXSMM_NBITS08(N))
 #define LIBXSMM_NBITS32(N) (0 != ((N) & 0xFFFF0000) ? (16 + LIBXSMM_NBITS16((N) >> 16)) : LIBXSMM_NBITS16(N))
 #define LIBXSMM_NBITS64(N) (0 != ((N) & 0xFFFFFFFF00000000) ? (32 + LIBXSMM_NBITS32((N) >> 32)) : LIBXSMM_NBITS32(N))
-#define LIBXSMM_NBITS(N) (0 != (N) ? (LIBXSMM_NBITS64((uint64_t)(N)) + 1) : 1)
+#define LIBXSMM_NBITS(N) (0 != (N) ? (LIBXSMM_NBITS64((unsigned long long)(N)) + 1) : 1)
 
+#define LIBXSMM_DEFAULT(DEFAULT, VALUE) (0 < (VALUE) ? (VALUE) : (DEFAULT))
 #define LIBXSMM_MIN(A, B) ((A) < (B) ? (A) : (B))
 #define LIBXSMM_MAX(A, B) ((A) < (B) ? (B) : (A))
 #define LIBXSMM_MOD2(N, NPOT) ((N) & ((NPOT) - 1))
@@ -174,7 +178,7 @@
 # define LIBXSMM_ASSUME_ALIGNED(A, N)
 # if defined(_MSC_VER)
 #   define LIBXSMM_ASSUME(EXPRESSION) __assume(EXPRESSION)
-# elif (40500 <= (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__))
+# elif (LIBXSMM_VERSION3(4, 5, 0) <= LIBXSMM_VERSION3(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__))
 #   define LIBXSMM_ASSUME(EXPRESSION) do { if (!(EXPRESSION)) __builtin_unreachable(); } while(0)
 # else
 #   define LIBXSMM_ASSUME(EXPRESSION)
@@ -182,31 +186,60 @@
 #endif
 #define LIBXSMM_ALIGN_VALUE(N, TYPESIZE, ALIGNMENT/*POT*/) (LIBXSMM_UP2((N) * (TYPESIZE), ALIGNMENT) / (TYPESIZE))
 #define LIBXSMM_ALIGN_VALUE2(N, POTSIZE, ALIGNMENT/*POT*/) LIBXSMM_DIV2(LIBXSMM_UP2(LIBXSMM_MUL2(N, POTSIZE), ALIGNMENT), POTSIZE)
-#define LIBXSMM_ALIGN(POINTER, ALIGNMENT/*POT*/) ((POINTER) + (LIBXSMM_ALIGN_VALUE((uintptr_t)(POINTER), 1, ALIGNMENT) - ((uintptr_t)(POINTER))) / sizeof(*(POINTER)))
-#define LIBXSMM_ALIGN2(POINTPOT, ALIGNMENT/*POT*/) ((POINTPOT) + LIBXSMM_DIV2(LIBXSMM_ALIGN_VALUE2((uintptr_t)(POINTPOT), 1, ALIGNMENT) - ((uintptr_t)(POINTPOT)), sizeof(*(POINTPOT))))
+#define LIBXSMM_ALIGN(POINTER, ALIGNMENT/*POT*/) ((POINTER) + (LIBXSMM_ALIGN_VALUE((unsigned long long)(POINTER), 1, ALIGNMENT) - ((unsigned long long)(POINTER))) / sizeof(*(POINTER)))
+#define LIBXSMM_ALIGN2(POINTPOT, ALIGNMENT/*POT*/) ((POINTPOT) + LIBXSMM_DIV2(LIBXSMM_ALIGN_VALUE2((unsigned long long)(POINTPOT), 1, ALIGNMENT) - ((unsigned long long)(POINTPOT)), sizeof(*(POINTPOT))))
 
 #define LIBXSMM_HASH_VALUE(N) ((((N) ^ ((N) >> 12)) ^ (((N) ^ ((N) >> 12)) << 25)) ^ ((((N) ^ ((N) >> 12)) ^ (((N) ^ ((N) >> 12)) << 25)) >> 27))
-#define LIBXSMM_HASH2(POINTER, ALIGNMENT/*POT*/, NPOT) LIBXSMM_MOD2(LIBXSMM_HASH_VALUE(LIBXSMM_DIV2((uintptr_t)(POINTER), ALIGNMENT)), NPOT)
+#define LIBXSMM_HASH2(POINTER, ALIGNMENT/*POT*/, NPOT) LIBXSMM_MOD2(LIBXSMM_HASH_VALUE(LIBXSMM_DIV2((unsigned long long)(POINTER), ALIGNMENT)), NPOT)
 
-#if defined(_WIN32) && !defined(__GNUC__)
-# define LIBXSMM_TLS LIBXSMM_ATTRIBUTE(thread)
-#elif defined(__GNUC__) || defined(__clang__)
-# define LIBXSMM_TLS __thread
-#elif defined(__cplusplus)
-# define LIBXSMM_TLS thread_local
+#if !defined(LIBXSMM_UNUSED)
+# if 0 /*defined(__GNUC__) && !defined(__clang__) && !defined(__INTEL_COMPILER)*/
+#   define LIBXSMM_UNUSED(VARIABLE) LIBXSMM_PRAGMA(unused(VARIABLE))
+# else
+#   define LIBXSMM_UNUSED(VARIABLE) (void)(VARIABLE)
+# endif
 #endif
 
-#if defined(__INTEL_OFFLOAD) && (!defined(_WIN32) || (1400 <= __INTEL_COMPILER))
-# define LIBXSMM_OFFLOAD_BUILD 1
-# define LIBXSMM_OFFLOAD(A) LIBXSMM_ATTRIBUTE(target(A))
+#if defined(__GNUC__) || (defined(__INTEL_COMPILER) && !defined(_WIN32))
+# define LIBXSMM_UNUSED_ARG LIBXSMM_ATTRIBUTE(unused)
 #else
-/*# define LIBXSMM_OFFLOAD_BUILD 0*/
-# define LIBXSMM_OFFLOAD(A)
+# define LIBXSMM_UNUSED_ARG
 #endif
-#if !defined(LIBXSMM_OFFLOAD_TARGET)
-# define LIBXSMM_OFFLOAD_TARGET mic
+
+#if defined(LIBXSMM_NOSYNC)
+# undef _REENTRANT
+#elif !defined(_REENTRANT)
+# define _REENTRANT
+#endif
+
+#if defined(_REENTRANT)
+# if (defined(_WIN32) && !defined(__GNUC__)) || defined(__clang__)
+#   define LIBXSMM_TLS LIBXSMM_ATTRIBUTE(thread)
+# elif defined(__GNUC__)
+#   define LIBXSMM_TLS __thread
+# elif defined(__cplusplus)
+#   define LIBXSMM_TLS thread_local
+# else
+#   error Missing TLS support!
+# endif
+#else
+# define LIBXSMM_TLS
+#endif
+
+#if defined(__GNUC__)
+# define LIBXSMM_VISIBILITY_HIDDEN LIBXSMM_ATTRIBUTE(visibility("hidden"))
+# define LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_ATTRIBUTE(visibility("internal"))
+#else
+# define LIBXSMM_VISIBILITY_HIDDEN
+# define LIBXSMM_VISIBILITY_INTERNAL
+#endif
+
+#if defined(NDEBUG)
+# define LIBXSMM_NDEBUG NDEBUG
+# define LIBXSMM_DEBUG(...)
+#else
+# define LIBXSMM_DEBUG(...) __VA_ARGS__
 #endif
-#define LIBXSMM_RETARGETABLE LIBXSMM_OFFLOAD(LIBXSMM_OFFLOAD_TARGET)
 
 /** Execute the CPUID, and receive results (EAX, EBX, ECX, EDX) for requested FUNCTION. */
 #if defined(__GNUC__)
@@ -226,7 +259,7 @@
 /** Execute the XGETBV, and receive results (EAX, EDX) for req. eXtended Control Register (XCR). */
 #if defined(__GNUC__)
 # define LIBXSMM_XGETBV(XCR, EAX, EDX) __asm__ __volatile__( \
-    "xgetbv" : "=a"(EAX), "=d"(EDX) : "c"(XCR) \
+    ".byte 0x0f, 0x01, 0xd0" /*xgetbv*/ : "=a"(EAX), "=d"(EDX) : "c"(XCR) \
   )
 #else
 # define LIBXSMM_XGETBV(XCR, EAX, EDX) { \
@@ -236,24 +269,6 @@
   }
 #endif
 
-/**
- * Below group of preprocessor symbols are used to fixup some platform specifics.
- */
-#if !defined(_CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES)
-# define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
-#endif
-#if !defined(_CRT_SECURE_NO_DEPRECATE)
-# define _CRT_SECURE_NO_DEPRECATE 1
-#endif
-#if !defined(_USE_MATH_DEFINES)
-# define _USE_MATH_DEFINES 1
-#endif
-#if !defined(WIN32_LEAN_AND_MEAN)
-# define WIN32_LEAN_AND_MEAN 1
-#endif
-#if !defined(NOMINMAX)
-# define NOMINMAX 1
-#endif
 #if defined(_WIN32)
 # define LIBXSMM_SNPRINTF(S, N, ...) _snprintf_s(S, N, _TRUNCATE, __VA_ARGS__)
 # define LIBXSMM_FLOCK(FILE) _lock_file(FILE)
@@ -273,33 +288,56 @@
 # endif
 #endif
 
-#if defined(__GNUC__)
-# if defined(LIBXSMM_OFFLOAD_BUILD)
-#   pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-#   include <pthread.h>
-#   pragma offload_attribute(pop)
-# else
-#   include <pthread.h>
-# endif
-# define LIBXSMM_LOCK_TYPE pthread_mutex_t
-# define LIBXSMM_LOCK_CONSTRUCT PTHREAD_MUTEX_INITIALIZER
-# define LIBXSMM_LOCK_DESTROY(LOCK) pthread_mutex_destroy(&(LOCK))
-# define LIBXSMM_LOCK_ACQUIRE(LOCK) pthread_mutex_lock(&(LOCK))
-# define LIBXSMM_LOCK_RELEASE(LOCK) pthread_mutex_unlock(&(LOCK))
-#else /*TODO: Windows*/
+#if defined(_WIN32) /*TODO*/
 # define LIBXSMM_LOCK_TYPE HANDLE
 # define LIBXSMM_LOCK_CONSTRUCT 0
 # define LIBXSMM_LOCK_DESTROY(LOCK) CloseHandle(LOCK)
 # define LIBXSMM_LOCK_ACQUIRE(LOCK) WaitForSingleObject(LOCK, INFINITE)
 # define LIBXSMM_LOCK_RELEASE(LOCK) ReleaseMutex(LOCK)
+#else /* PThreads: include <pthread.h> */
+# define LIBXSMM_LOCK_TYPE pthread_mutex_t
+# define LIBXSMM_LOCK_CONSTRUCT PTHREAD_MUTEX_INITIALIZER
+# define LIBXSMM_LOCK_DESTROY(LOCK) pthread_mutex_destroy(&(LOCK))
+# define LIBXSMM_LOCK_ACQUIRE(LOCK) pthread_mutex_lock(&(LOCK))
+# define LIBXSMM_LOCK_RELEASE(LOCK) pthread_mutex_unlock(&(LOCK))
 #endif
 
-#if defined(LIBXSMM_OFFLOAD_BUILD)
-# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-# include <stdint.h>
-# pragma offload_attribute(pop)
-#else
-# include <stdint.h>
+/** Below group is to fixup some platform/compiler specifics. */
+#if defined(_WIN32)
+# if !defined(_CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES)
+#   define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
+# endif
+# if !defined(_CRT_SECURE_NO_DEPRECATE)
+#   define _CRT_SECURE_NO_DEPRECATE 1
+# endif
+# if !defined(_USE_MATH_DEFINES)
+#   define _USE_MATH_DEFINES 1
+# endif
+# if !defined(WIN32_LEAN_AND_MEAN)
+#   define WIN32_LEAN_AND_MEAN 1
+# endif
+# if !defined(NOMINMAX)
+#   define NOMINMAX 1
+# endif
+# if defined(__INTEL_COMPILER) && (190023506 <= _MSC_FULL_VER)
+#   define __builtin_huge_val() HUGE_VAL
+#   define __builtin_huge_valf() HUGE_VALF
+#   define __builtin_nan nan
+#   define __builtin_nanf nanf
+#   define __builtin_nans nan
+#   define __builtin_nansf nanf
+# endif
+#endif
+#if defined(__GNUC__)
+# if !defined(_GNU_SOURCE)
+#   define _GNU_SOURCE
+# endif
+#endif
+#if defined(__clang__)
+# if !defined(__extern_always_inline)
+#   define __extern_always_inline LIBXSMM_INLINE_KEYWORD
+# endif
 #endif
 
 #endif /*LIBXSMM_MACROS_H*/
+
