@@ -1,5 +1,5 @@
 !*****************************************************************************!
-!* Copyright (c) 2013-2015, Intel Corporation                                *!
+!* Copyright (c) 2015-2016, Intel Corporation                                *!
 !* All rights reserved.                                                      *!
 !*                                                                           *!
 !* Redistribution and use in source and binary forms, with or without        *!
@@ -48,10 +48,10 @@ PROGRAM stpm
   !DIR$ ATTRIBUTES ALIGN:LIBXSMM_ALIGNMENT :: tm1, tm2, tm3 
   !$OMP THREADPRIVATE(tm1, tm2, tm3)
   TYPE(LIBXSMM_DMMFUNCTION) :: xmm1, xmm2, xmm3
-  INTEGER :: argc, m, n, k, routine, check
-  INTEGER(8) :: i, j, s, ix, iy, iz, start, reps, r, totsize
-  CHARACTER(32) :: argv
   DOUBLE PRECISION :: duration, max_diff, h1, h2
+  INTEGER :: argc, m, n, k, routine, check
+  INTEGER(8) :: i, j, ix, iy, iz, r, s, total_size, repetitions, start
+  CHARACTER(32) :: argv
 
   argc = COMMAND_ARGUMENT_COUNT()
   IF (1 <= argc) THEN
@@ -76,27 +76,22 @@ PROGRAM stpm
     CALL GET_COMMAND_ARGUMENT(4, argv)
     READ(argv, "(I32)") i
   ELSE
-    i = 2 ! 2 GByte for A and B (and C, but this currently not used by the F90 test)
+    i = 0
   END IF
   IF (5 <= argc) THEN
     CALL GET_COMMAND_ARGUMENT(5, argv)
-    READ(argv, "(I32)") totsize
+    READ(argv, "(I32)") total_size
   ELSE
-    totsize = i ! -> we have 1 iteration by default
-  END IF
-
-  ! determining how many repitions are needed
-  IF (i >= totsize) THEN
-    reps = 1
-    totsize = i
-  ELSE
-    reps = totsize / i
+    total_size = 0 ! 1 repetition by default
   END IF
 
   ! Initialize LIBXSMM
   CALL libxsmm_init()
 
-  s = ISHFT(MAX(i, 0_8), 30) / ((m * n * k) * T * 6)
+  ! workload is about 2 GByte in memory by default
+  s = MERGE(ISHFT(2_8, 30) / ((m * n * k) * T * 6), MAX(i, 0_8), 0.EQ.i)
+  ! determining how many repititions are needed
+  total_size = MAX(s, total_size); repetitions = total_size / s
   duration = 0; max_diff = 0
 
   ALLOCATE(a(m,n,k,s))
@@ -124,8 +119,8 @@ PROGRAM stpm
   dx = 1.; dy = 1.; dz = 1.
   h1 = 1.; h2 = 1.
 
-  WRITE(*, "(A,I0,A,I0,A,I0,A,I0,A,I0,A,I0)") "m=", m, " n=", n, " k=", k, " size=", UBOUND(a, 4), &
-  " total-stream-GB=", totsize, " reps=", reps
+  WRITE(*, "(3(A,I0),A,I0,A,I0)") "m=", m, " n=", n, " k=", k, &
+    " size=", UBOUND(a, 4), " repetitions=", repetitions
 
   CALL GETENV("CHECK", argv)
   READ(argv, "(I32)") check
@@ -144,11 +139,11 @@ PROGRAM stpm
       END DO
     END DO 
 
-    !$OMP PARALLEL PRIVATE(i) DEFAULT(NONE) &
-    !$OMP   SHARED(duration, a, b, dx, dy, dz, g1, g2, g3, d, m, n, k, h1, h2, reps)
+    !$OMP PARALLEL PRIVATE(i, j, r) DEFAULT(NONE) &
+    !$OMP   SHARED(a, b, dx, dy, dz, g1, g2, g3, d, m, n, k, h1, h2, repetitions)
     ALLOCATE(tm1(m,n,k), tm2(m,n,k), tm3(m*n,k,1))
     tm1 = 0; tm2 = 0; tm3 = 0
-    DO r = 1, reps
+    DO r = 1, repetitions
       !$OMP DO
       DO i = LBOUND(a, 4), UBOUND(a, 4)
         CALL libxsmm_blas_gemm(m=m, n=n*k, k=m, a=dx, b=a(:,:,1,i), c=tm1(:,:,1), alpha=alpha, beta=beta)
@@ -168,14 +163,15 @@ PROGRAM stpm
 
   c(:,:,:,:) = 0.0
   WRITE(*, "(A)") "Streamed... (BLAS)"
-  !$OMP PARALLEL PRIVATE(i, start) DEFAULT(NONE) &
-  !$OMP   SHARED(duration, a, dx, dy, dz, g1, g2, g3, b, c, m, n, k, h1, h2, reps)
+  !$OMP PARALLEL PRIVATE(i, j, r, start) DEFAULT(NONE) &
+  !$OMP   SHARED(a, dx, dy, dz, g1, g2, g3, b, c, m, n, k, h1, h2, duration, repetitions)
   ALLOCATE(tm1(m,n,k), tm2(m,n,k), tm3(m,n,k))
   tm1 = 0; tm2 = 0; tm3 = 0
   !$OMP MASTER
   start = libxsmm_timer_tick()
   !$OMP END MASTER
-  DO r = 1, reps
+  !$OMP BARRIER
+  DO r = 1, repetitions
     !$OMP DO
     DO i = LBOUND(a, 4), UBOUND(a, 4)
       CALL libxsmm_blas_gemm(m=m, n=n*k, k=m, a=dx, b=a(:,:,1,i), c=tm1(:,:,1), alpha=alpha, beta=beta)
@@ -189,6 +185,7 @@ PROGRAM stpm
                                     h1, h2, m*n*k )
     END DO
   END DO
+  !$OMP BARRIER
   !$OMP MASTER
   duration = libxsmm_timer_duration(start, libxsmm_timer_tick())
   !$OMP END MASTER
@@ -197,19 +194,20 @@ PROGRAM stpm
   !$OMP END PARALLEL
 
   ! Print Performance Summary and check results
-  CALL performance(duration, m, n, k, s, reps)
+  CALL performance(duration, m, n, k, total_size)
   IF (check.NE.0) max_diff = MAX(max_diff, validate(d, c))
 
   c(:,:,:,:) = 0.0
   WRITE(*, "(A)") "Streamed... (mxm)"
-  !$OMP PARALLEL PRIVATE(i, start) DEFAULT(NONE) &
-  !$OMP   SHARED(duration, a, dx, dy, dz, g1, g2, g3, b, c, m, n, k, h1, h2, reps)
+  !$OMP PARALLEL PRIVATE(i, j, r, start) DEFAULT(NONE) &
+  !$OMP   SHARED(a, dx, dy, dz, g1, g2, g3, b, c, m, n, k, h1, h2, duration, repetitions)
   ALLOCATE(tm1(m,n,k), tm2(m,n,k), tm3(m,n,k))
   tm1 = 0; tm2 = 0; tm3 = 0
   !$OMP MASTER
   start = libxsmm_timer_tick()
   !$OMP END MASTER
-  DO r = 1, reps
+  !$OMP BARRIER
+  DO r = 1, repetitions
     !$OMP DO
     DO i = LBOUND(a, 4), UBOUND(a, 4)
       CALL mxmf2(dx, m, a(:,:,:,i), m, tm1, n*k)
@@ -223,6 +221,7 @@ PROGRAM stpm
                                     h1, h2, m*n*k )
     END DO
   END DO
+  !$OMP BARRIER
   !$OMP MASTER
   duration = libxsmm_timer_duration(start, libxsmm_timer_tick())
   !$OMP END MASTER
@@ -231,19 +230,20 @@ PROGRAM stpm
   !$OMP END PARALLEL
 
   ! Print Performance Summary and check results
-  CALL performance(duration, m, n, k, s, reps)
+  CALL performance(duration, m, n, k, total_size)
   IF (check.NE.0) max_diff = MAX(max_diff, validate(d, c))
 
   c(:,:,:,:) = 0.0
   WRITE(*, "(A)") "Streamed... (auto-dispatched)"
-  !$OMP PARALLEL PRIVATE(i, start) DEFAULT(NONE) &
-  !$OMP   SHARED(duration, a, b, dx, dy, dz, g1, g2, g3, c, m, n, k, h1, h2, reps)
+  !$OMP PARALLEL PRIVATE(i, j, r, start) DEFAULT(NONE) &
+  !$OMP   SHARED(a, b, dx, dy, dz, g1, g2, g3, c, m, n, k, h1, h2, duration, repetitions)
   ALLOCATE(tm1(m,n,k), tm2(m,n,k), tm3(m,n,k))
   tm1 = 0; tm2 = 0; tm3 = 0
   !$OMP MASTER
   start = libxsmm_timer_tick()
   !$OMP END MASTER
-  DO r = 1, reps
+  !$OMP BARRIER
+  DO r = 1, repetitions
     !$OMP DO
     DO i = LBOUND(a, 4), UBOUND(a, 4)
       CALL libxsmm_gemm(m=m, n=n*k, k=m, a=dx, b=a(:,:,1,i), c=tm1(:,:,1), alpha=alpha, beta=beta)
@@ -257,6 +257,7 @@ PROGRAM stpm
                                     h1, h2, m*n*k )
     END DO
   END DO
+  !$OMP BARRIER
   !$OMP MASTER
   duration = libxsmm_timer_duration(start, libxsmm_timer_tick())
   !$OMP END MASTER
@@ -265,7 +266,7 @@ PROGRAM stpm
   !$OMP END PARALLEL
 
   ! Print Performance Summary and check results
-  CALL performance(duration, m, n, k, s, reps)
+  CALL performance(duration, m, n, k, total_size)
   IF (check.NE.0) max_diff = MAX(max_diff, validate(d, c))
 
   c(:,:,:,:) = 0.0
@@ -274,13 +275,14 @@ PROGRAM stpm
   CALL libxsmm_dispatch(xmm2, m, n, n, alpha=alpha, beta=beta)
   CALL libxsmm_dispatch(xmm3, m*n, k, k, alpha=alpha, beta=beta)
   IF (libxsmm_available(xmm1).AND.libxsmm_available(xmm2).AND.libxsmm_available(xmm3)) THEN
-    !$OMP PARALLEL PRIVATE(i, start) !DEFAULT(NONE) SHARED(duration, a, dx, dy, dz, g1, g2, g3, b, c, m, n, k, xmm1, xmm2, xmm3, h1, h2, reps)
+    !$OMP PARALLEL PRIVATE(i, j, r, start) !DEFAULT(NONE) SHARED(a, dx, dy, dz, g1, g2, g3, b, c, m, n, k, h1, h2, duration, repetitions, xmm1, xmm2, xmm3)
     ALLOCATE(tm1(m,n,k), tm2(m,n,k), tm3(m,n,k))
     tm1 = 0; tm2 = 0; tm3 = 0
     !$OMP MASTER
     start = libxsmm_timer_tick()
     !$OMP END MASTER
-    DO r = 1, reps
+    !$OMP BARRIER
+    DO r = 1, repetitions
       !$OMP DO
       DO i = LBOUND(a, 4), UBOUND(a, 4)
         CALL libxsmm_call(xmm1, C_LOC(dx), C_LOC(a(1,1,1,i)), C_LOC(tm1))
@@ -294,6 +296,7 @@ PROGRAM stpm
                                       h1, h2, m*n*k )
       END DO
     END DO
+    !$OMP BARRIER
     !$OMP MASTER
     duration = libxsmm_timer_duration(start, libxsmm_timer_tick())
     !$OMP END MASTER
@@ -302,7 +305,7 @@ PROGRAM stpm
     !$OMP END PARALLEL
 
     ! Print Performance Summary and check results
-    CALL performance(duration, m, n, k, s, reps)
+    CALL performance(duration, m, n, k, total_size)
     IF (check.NE.0) max_diff = MAX(max_diff, validate(d, c))
   ELSE
     WRITE(*,*) "Could not build specialized function(s)!"
@@ -331,17 +334,16 @@ CONTAINS
     WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "diff:       ", diff
   END FUNCTION
 
-  SUBROUTINE performance(duration, m, n, k, s, reps)
-    DOUBLE PRECISION, INTENT(IN)    :: duration
+  SUBROUTINE performance(duration, m, n, k, size)
+    DOUBLE PRECISION, INTENT(IN) :: duration
     INTEGER, INTENT(IN)    :: m, n, k
-    INTEGER(8), INTENT(IN) :: s, reps
-
+    INTEGER(8), INTENT(IN) :: size
     IF (0.LT.duration) THEN
       WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "performance:", &
-        (reps * s * m * n * k * (2*(m+n+k) + 2 + 4) * 1D-9 / duration), " GFLOPS/s"
+        (size * m * n * k * (2*(m+n+k) + 2 + 4) * 1D-9 / duration), " GFLOPS/s"
       WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "bandwidth:  ", &
-        (reps * s * m * n * k * (6) * T / (duration * ISHFT(1_8, 30))), " GB/s"
+        (size * m * n * k * (6) * T / (duration * ISHFT(1_8, 30))), " GB/s"
     END IF
-    WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "duration:   ", (1D3 * duration)/reps, " ms"
+    WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "duration:   ", (1D3 * duration)/repetitions, " ms"
   END SUBROUTINE
 END PROGRAM
