@@ -1,5 +1,5 @@
 !*****************************************************************************!
-!* Copyright (c) 2013-2015, Intel Corporation                                *!
+!* Copyright (c) 2015-2016, Intel Corporation                                *!
 !* All rights reserved.                                                      *!
 !*                                                                           *!
 !* Redistribution and use in source and binary forms, with or without        *!
@@ -47,11 +47,10 @@ PROGRAM stpm
   !DIR$ ATTRIBUTES ALIGN:LIBXSMM_ALIGNMENT :: a, c, d
   !$OMP THREADPRIVATE(tm1, tm2, tm3)
   TYPE(LIBXSMM_DMMFUNCTION) :: xmm1, xmm2, xmm3
-  INTEGER :: argc, m, n, k, routine, check
-  integer :: mm, nn, kk
-  INTEGER(8) :: i, j, s, ix, iy, iz, start, reps, r, totsize
-  CHARACTER(32) :: argv
   DOUBLE PRECISION :: duration, max_diff
+  INTEGER :: argc, m, n, k, routine, check, mm, nn, kk
+  INTEGER(8) :: i, j, ix, iy, iz, r, s, total_size, repetitions, start
+  CHARACTER(32) :: argv
 
   argc = COMMAND_ARGUMENT_COUNT()
   IF (1 <= argc) THEN
@@ -72,49 +71,44 @@ PROGRAM stpm
   ELSE
     k = m
   END IF
+  mm = 0
   IF (4 <= argc) THEN
     CALL GET_COMMAND_ARGUMENT(4, argv)
     READ(argv, "(I32)") mm
-  ELSE
-    mm = 10
   END IF
+  mm = MERGE(10, mm, 0.EQ.mm)
+  nn = 0
   IF (5 <= argc) THEN
     CALL GET_COMMAND_ARGUMENT(5, argv)
     READ(argv, "(I32)") nn
-  ELSE
-    nn = mm
   END IF
+  nn = MERGE(mm, nn, 0.EQ.nn)
+  kk = 0
   IF (6 <= argc) THEN
     CALL GET_COMMAND_ARGUMENT(6, argv)
     READ(argv, "(I32)") kk
-  ELSE
-    kk = mm
   END IF
+  kk = MERGE(mm, kk, 0.EQ.kk)
   IF (7 <= argc) THEN
     CALL GET_COMMAND_ARGUMENT(7, argv)
     READ(argv, "(I32)") i
   ELSE
-    i = 2 ! 2 GByte for A and B (and C, but this currently not used by the F90 test)
+    i = 0
   END IF
   IF (8 <= argc) THEN
     CALL GET_COMMAND_ARGUMENT(8, argv)
-    READ(argv, "(I32)") totsize
+    READ(argv, "(I32)") total_size
   ELSE
-    totsize = i ! -> we have 1 iteration by default
-  END IF
-
-  ! determining how many repitions are needed
-  IF (i >= totsize) THEN
-    reps = 1
-    totsize = i
-  ELSE
-    reps = totsize / i
+    total_size = 0 ! 1 repetition by default
   END IF
 
   ! Initialize LIBXSMM
   CALL libxsmm_init()
 
-  s = ISHFT(MAX(i, 0_8), 29) / (((m * n * k) + (nn * mm * kk)) * T)
+  ! workload is about 2 GByte in memory by default
+  s = MERGE(ISHFT(2_8, 30) / (((m * n * k) + (nn * mm * kk)) * T), MAX(i, 0_8), 0.EQ.i)
+  ! determining how many repititions are needed
+  total_size = MAX(s, total_size); repetitions = total_size / s
   duration = 0; max_diff = 0
 
   ALLOCATE(a(m,n,k,s))
@@ -122,7 +116,7 @@ PROGRAM stpm
   ALLOCATE(dx(mm,m), dy(n,nn), dz(k,kk))
 
   ! Initialize
-  !$OMP PARALLEL DO PRIVATE(i) DEFAULT(NONE) SHARED(a, m, mm, n, nn, k, kk, s)
+  !$OMP PARALLEL DO PRIVATE(i, ix, iy, iz) DEFAULT(NONE) SHARED(a, m, mm, n, nn, k, kk, s)
   DO i = 1, s
     DO ix = 1, m
       DO iy = 1, n
@@ -132,7 +126,7 @@ PROGRAM stpm
       END DO
     END DO
   END DO 
-  !$OMP PARALLEL DO PRIVATE(i) DEFAULT(NONE) SHARED(c, m, mm, n, nn, k, kk, s)
+  !$OMP PARALLEL DO PRIVATE(i, ix, iy, iz) DEFAULT(NONE) SHARED(c, m, mm, n, nn, k, kk, s)
   DO i = 1, s
     DO ix = 1, mm
       DO iy = 1, nn
@@ -146,14 +140,14 @@ PROGRAM stpm
   dy = 1.
   dz = 1.
 
-  WRITE(*, "(6(A,I0),A,I0,A,I0,A,I0)") "m=", m, " n=", n, " k=", k, " mm=", mm, " nn=", nn, &
-  " kk=", kk, " size=", UBOUND(a, 4), " total-stream-GB=", totsize, " reps=", reps
+  WRITE(*, "(6(A,I0),A,I0,A,I0)") "m=", m, " n=", n, " k=", k, &
+    " mm=", mm, " nn=", nn, " kk=", kk, " size=", UBOUND(a, 4), " repetitions=", repetitions
 
   CALL GETENV("CHECK", argv)
   READ(argv, "(I32)") check
   IF (0.NE.check) THEN
     ALLOCATE(d(mm,nn,kk,s))
-    !$OMP PARALLEL DO PRIVATE(i) DEFAULT(NONE) SHARED(d, m, mm, n, nn, k, kk, s)
+    !$OMP PARALLEL DO PRIVATE(i, ix, iy, iz) DEFAULT(NONE) SHARED(d, m, mm, n, nn, k, kk, s)
     DO i = 1, s
       DO ix = 1, mm
         DO iy = 1, nn
@@ -165,11 +159,11 @@ PROGRAM stpm
     END DO 
 
     WRITE(*, "(A)") "Calculating check..."
-    !$OMP PARALLEL PRIVATE(i) DEFAULT(NONE) &
-    !$OMP   SHARED(duration, a, dx, dy, dz, d, m, n, k, mm, nn, kk, reps)
+    !$OMP PARALLEL PRIVATE(i, j, r) DEFAULT(NONE) &
+    !$OMP   SHARED(a, dx, dy, dz, d, m, n, k, mm, nn, kk, repetitions)
     ALLOCATE(tm1(mm,n,k), tm2(mm,nn,k))
     tm1 = 0; tm2 = 0;
-    DO r = 1, reps
+    DO r = 1, repetitions
       !$OMP DO
       DO i = LBOUND(a, 4), UBOUND(a, 4)
         tm1 = reshape(matmul(dx, reshape(a(:,:,:,i), (/m,n*k/))), (/mm, n, k/))
@@ -186,14 +180,15 @@ PROGRAM stpm
   END IF
 
   WRITE(*, "(A)") "Streamed... (BLAS)"
-  !$OMP PARALLEL PRIVATE(i, start) DEFAULT(NONE) &
-  !$OMP   SHARED(duration, a, dx, dy, dz, c, m, n, k, mm, nn, kk, reps)
+  !$OMP PARALLEL PRIVATE(i, j, r, start) DEFAULT(NONE) &
+  !$OMP   SHARED(a, dx, dy, dz, c, m, n, k, mm, nn, kk, duration, repetitions)
   ALLOCATE(tm1(mm,n,k), tm2(mm,nn,k), tm3(mm,nn,kk))
   tm1 = 0; tm2 = 0; tm3 = 3
   !$OMP MASTER
   start = libxsmm_timer_tick()
   !$OMP END MASTER
-  DO r = 1, reps
+  !$OMP BARRIER
+  DO r = 1, repetitions
     !$OMP DO
     DO i = LBOUND(a, 4), UBOUND(a, 4)
       CALL libxsmm_blas_gemm(m=mm, n=n*k, k=m, a=dx, b=a(:,:,1,i), c=tm1(:,:,1), alpha=alpha, beta=beta)
@@ -204,6 +199,7 @@ PROGRAM stpm
       CALL stream_vector_copy( tm3(1,1,1), c(1,1,1,i), mm*nn*kk )
     END DO
   END DO
+  !$OMP BARRIER
   !$OMP MASTER
   duration = libxsmm_timer_duration(start, libxsmm_timer_tick())
   !$OMP END MASTER
@@ -211,18 +207,19 @@ PROGRAM stpm
   DEALLOCATE(tm1, tm2, tm3)
   !$OMP END PARALLEL
 
-  CALL performance(duration, m, n, k, mm, nn, kk, s, reps)
+  CALL performance(duration, m, n, k, mm, nn, kk, total_size)
   IF (check.NE.0) max_diff = MAX(max_diff, validate(c, d))
 
   WRITE(*, "(A)") "Streamed... (mxm)"
-  !$OMP PARALLEL PRIVATE(i, start) DEFAULT(NONE) &
-  !$OMP   SHARED(duration, a, dx, dy, dz, c, m, n, k, mm, nn, kk, reps)
+  !$OMP PARALLEL PRIVATE(i, j, r, start) DEFAULT(NONE) &
+  !$OMP   SHARED(a, dx, dy, dz, c, m, n, k, mm, nn, kk, duration, repetitions)
   ALLOCATE(tm1(mm,n,k), tm2(mm,nn,k), tm3(mm,nn,kk))
   tm1 = 0; tm2 = 0; tm3 = 3
   !$OMP MASTER
   start = libxsmm_timer_tick()
   !$OMP END MASTER
-  DO r = 1, reps
+  !$OMP BARRIER
+  DO r = 1, repetitions
     !$OMP DO
     DO i = LBOUND(a, 4), UBOUND(a, 4)
       CALL mxmf2(dx, mm, a(:,:,:,i), m, tm1, n*k)
@@ -233,6 +230,7 @@ PROGRAM stpm
       CALL stream_vector_copy( tm3(1,1,1), c(1,1,1,i), mm*nn*kk )
     END DO
   END DO
+  !$OMP BARRIER
   !$OMP MASTER
   duration = libxsmm_timer_duration(start, libxsmm_timer_tick())
   !$OMP END MASTER
@@ -240,18 +238,19 @@ PROGRAM stpm
   DEALLOCATE(tm1, tm2, tm3)
   !$OMP END PARALLEL
 
-  CALL performance(duration, m, n, k, mm, nn, kk, s, reps)
+  CALL performance(duration, m, n, k, mm, nn, kk, total_size)
   IF (check.NE.0) max_diff = MAX(max_diff, validate(c, d))
 
   WRITE(*, "(A)") "Streamed... (auto-dispatched)"
-  !$OMP PARALLEL PRIVATE(i, start) DEFAULT(NONE) &
-  !$OMP   SHARED(duration, a, dx, dy, dz, c, m, n, k, mm, nn, kk, reps)
+  !$OMP PARALLEL PRIVATE(i, j, r, start) DEFAULT(NONE) &
+  !$OMP   SHARED(a, dx, dy, dz, c, m, n, k, mm, nn, kk, duration, repetitions)
   ALLOCATE(tm1(mm,n,k), tm2(mm,nn,k), tm3(mm,nn,kk))
   tm1 = 0; tm2 = 0; tm3 = 3
   !$OMP MASTER
   start = libxsmm_timer_tick()
   !$OMP END MASTER
-  DO r = 1, reps
+  !$OMP BARRIER
+  DO r = 1, repetitions
     !$OMP DO
     DO i = LBOUND(a, 4), UBOUND(a, 4)
       CALL libxsmm_gemm(m=mm, n=n*k, k=m, a=dx, b=a(:,:,1,i), c=tm1(:,:,1), alpha=alpha, beta=beta)
@@ -262,6 +261,7 @@ PROGRAM stpm
       CALL stream_vector_copy( tm3(1,1,1), c(1,1,1,i), mm*nn*kk )
     END DO
   END DO
+  !$OMP BARRIER
   !$OMP MASTER
   duration = libxsmm_timer_duration(start, libxsmm_timer_tick())
   !$OMP END MASTER
@@ -269,7 +269,7 @@ PROGRAM stpm
   DEALLOCATE(tm1, tm2, tm3)
   !$OMP END PARALLEL
 
-  CALL performance(duration, m, n, k, mm, nn, kk, s, reps)
+  CALL performance(duration, m, n, k, mm, nn, kk, total_size)
   IF (check.NE.0) max_diff = MAX(max_diff, validate(c, d))
 
   WRITE(*, "(A)") "Streamed... (specialized)"
@@ -277,13 +277,14 @@ PROGRAM stpm
   CALL libxsmm_dispatch(xmm2, mm, nn, n, alpha=alpha, beta=beta)
   CALL libxsmm_dispatch(xmm3, mm*nn, kk, k, alpha=alpha, beta=beta)
   IF (libxsmm_available(xmm1).AND.libxsmm_available(xmm2).AND.libxsmm_available(xmm3)) THEN
-    !$OMP PARALLEL PRIVATE(i, start) !DEFAULT(NONE) SHARED(duration, a, dx, dy, dz, c, m, n, k, mm, nn, kk, xmm1, xmm2, xmm3, reps)
+    !$OMP PARALLEL PRIVATE(i, j, r, start) !DEFAULT(NONE) SHARED(a, dx, dy, dz, c, m, n, k, mm, nn, kk, duration, repetitions, xmm1, xmm2, xmm3)
     ALLOCATE(tm1(mm,n,k), tm2(mm,nn,k), tm3(mm,nn,kk))
     tm1 = 0; tm2 = 0; tm3 = 3
     !$OMP MASTER
     start = libxsmm_timer_tick()
     !$OMP END MASTER
-    DO r = 1, reps
+    !$OMP BARRIER
+    DO r = 1, repetitions
       !$OMP DO
       DO i = LBOUND(a, 4), UBOUND(a, 4)
         CALL libxsmm_call(xmm1, C_LOC(dx), C_LOC(a(1,1,1,i)), C_LOC(tm1))
@@ -294,6 +295,7 @@ PROGRAM stpm
         CALL stream_vector_copy( tm3(1,1,1), c(1,1,1,i), mm*nn*kk )
       END DO
     END DO
+    !$OMP BARRIER
     !$OMP MASTER
     duration = libxsmm_timer_duration(start, libxsmm_timer_tick())
     !$OMP END MASTER
@@ -301,7 +303,7 @@ PROGRAM stpm
     DEALLOCATE(tm1, tm2, tm3)
     !$OMP END PARALLEL
 
-    CALL performance(duration, m, n, k, mm, nn, kk, s, reps)
+    CALL performance(duration, m, n, k, mm, nn, kk, total_size)
     IF (check.NE.0) max_diff = MAX(max_diff, validate(c, d))
   ELSE
     WRITE(*,*) "Could not build specialized function(s)!"
@@ -325,17 +327,16 @@ CONTAINS
     WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "diff:       ", diff
   END FUNCTION
 
-  SUBROUTINE performance(duration, m, n, k, mm, nn, kk, s, reps)
-    real(8), intent(in) :: duration
-    integer, intent(in) :: m, n, k, mm, nn, kk
-    integer(8), intent(in) :: s, reps
-
+  SUBROUTINE performance(duration, m, n, k, mm, nn, kk, size)
+    DOUBLE PRECISION, INTENT(IN) :: duration
+    INTEGER, INTENT(IN)    :: m, n, k, mm, nn, kk
+    INTEGER(8), INTENT(IN) :: size
     IF (0.LT.duration) THEN
       WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "performance:", &
-        (reps * s * ((2*m-1)*mm*n*k + mm*(2*n-1)*nn*k + mm*nn*(2*k-1)*kk) * 1D-9 / duration), " GFLOPS/s"
+        (size * ((2*m-1)*mm*n*k + mm*(2*n-1)*nn*k + mm*nn*(2*k-1)*kk) * 1D-9 / duration), " GFLOPS/s"
       WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "bandwidth:  ", &
-        (reps * s * ((m*n*k) + (mm*nn*kk)) * T / (duration * LSHIFT(1_8, 30))), " GB/s"
+        (size * ((m*n*k) + (mm*nn*kk)) * T / (duration * LSHIFT(1_8, 30))), " GB/s"
     END IF
-    WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "duration:   ", (1D3 * duration)/reps, " ms"
+    WRITE(*, "(1A,A,F10.1,A)") CHAR(9), "duration:   ", (1D3 * duration)/repetitions, " ms"
   END SUBROUTINE
 END PROGRAM
