@@ -1,6 +1,6 @@
 ﻿#! /usr/bin/env python
 ###############################################################################
-## Copyright (c) 2013-2015, Intel Corporation                                ##
+## Copyright (c) 2014-2016, Intel Corporation                                ##
 ## All rights reserved.                                                      ##
 ##                                                                           ##
 ## Redistribution and use in source and binary forms, with or without        ##
@@ -41,13 +41,14 @@ if __name__ == "__main__":
         prefetch = int(sys.argv[5])
 
         mnkstr = str(m) + "_" + str(n) + "_" + str(k)
-        signature = "a, b, c" if (0 == prefetch) else "a, b, c, pa, pb, pc"
+        if (0 == prefetch): signature = "a, b, c"
+        else: signature = "a, b, c, pa, pb, pc"
         if (2 != precision):
             print
             print
             print("LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_smm_" + mnkstr + "(")
             print("  const float* a, const float* b, float* c" \
-               + (", const float* pa, const float* pb, const float* pc)" if (0 != prefetch) else ")"))
+               + [")", ", const float* pa, const float* pb, const float* pc)"][0!=prefetch])
             print("{")
             print("#if defined(__AVX512F__) && defined(LIBXSMM_GENTARGET_knl_sp)")
             print("  libxsmm_smm_" + mnkstr + "_knl(" + signature + ");")
@@ -60,9 +61,9 @@ if __name__ == "__main__":
             print("#elif defined(__MIC__) && defined(LIBXSMM_GENTARGET_knc_sp)")
             print("  libxsmm_smm_" + mnkstr + "_knc(" + signature + ");")
             print("#else")
-            print("  LIBXSMM_MESSAGE(\"===============================================================\")")
-            print("  LIBXSMM_MESSAGE(\"No specific instruction set extension found for specialization!\")")
-            print("  LIBXSMM_MESSAGE(\"===============================================================\")")
+            print("  LIBXSMM_MESSAGE(\"================================================================================\")")
+            print("  LIBXSMM_MESSAGE(\"LIBXSMM: No specific instruction set extension found for specialization!\")")
+            print("  LIBXSMM_MESSAGE(\"================================================================================\")")
             if (0 != prefetch):
                 print("  LIBXSMM_UNUSED(pa); LIBXSMM_UNUSED(pb); LIBXSMM_UNUSED(pc);")
             print("  LIBXSMM_INLINE_XGEMM(float, int/*libxsmm_blasint not req.*/, LIBXSMM_FSYMBOL(sgemm),\n" \
@@ -76,7 +77,7 @@ if __name__ == "__main__":
             print
             print("LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_dmm_" + mnkstr + "(")
             print("  const double* a, const double* b, double* c" \
-               + (", const double* pa, const double* pb, const double* pc)" if (0 != prefetch) else ")"))
+               + [")", ", const double* pa, const double* pb, const double* pc)"][0!=prefetch])
             print("{")
             print("#if defined(__AVX512F__) && defined(LIBXSMM_GENTARGET_knl_dp)")
             print("  libxsmm_dmm_" + mnkstr + "_knl(" + signature + ");")
