@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015, Intel Corporation                                     **
+** Copyright (c) 2015-2016, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -28,9 +28,10 @@
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
-#include "generator_dense_instructions.h"
+
 #include "generator_dense_common.h"
 #include "generator_common.h"
+#include "generator_x86_instructions.h"
 #include <libxsmm_macros.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -428,87 +429,6 @@ void libxsmm_generator_dense_init_micro_kernel_config_scalar( libxsmm_micro_kern
   io_micro_kernel_config->alu_mov_instruction = LIBXSMM_X86_INSTR_MOVQ;
 }
 
-void libxsmm_generator_dense_add_isa_check_header( libxsmm_generated_code* io_generated_code,
-                                                   const char*             i_arch ) {
-  if ( io_generated_code->code_type == 0 ) {
-    char l_new_code[512];
-    int l_max_code_length = 511;
-    int l_code_length = 0;
-
-    if ( (strcmp( i_arch, "wsm" ) == 0) ) {
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __SSE3__\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX__\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#pragma message (\"LIBXSMM KERNEL COMPILATION WARNING: compiling SSE3 code on AVX or newer architecture: \" __FILE__)\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#endif\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "snb" ) == 0) ) {
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX__\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX2__\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#pragma message (\"LIBXSMM KERNEL COMPILATION WARNING: compiling AVX code on AVX2 or newer architecture: \" __FILE__)\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#endif\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "hsw" ) == 0) ) {
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX2__\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX512F__\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#pragma message (\"LIBXSMM KERNEL COMPILATION WARNING: compiling AVX2 code on AVX512 or newer architecture: \" __FILE__)\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#endif\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "knl" ) == 0) ||
-                (strcmp( i_arch, "skx" ) == 0) ) {
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX512F__\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "knc" ) == 0) ) {
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __MIC__\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "noarch" ) == 0) ) {
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#pragma message (\"LIBXSMM KERNEL COMPILATION WARNING: compiling arch-independent gemm kernel in: \" __FILE__)\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
-      return;
-    }
-  }
-}
-
-void libxsmm_generator_dense_add_isa_check_footer( libxsmm_generated_code* io_generated_code,
-                                                   const char*             i_arch ) {
-  if ( io_generated_code->code_type == 0 ) {
-    char l_new_code[512];
-    int l_max_code_length = 511;
-    int l_code_length = 0;
-
-    if ( (strcmp( i_arch, "wsm" ) == 0) ||
-         (strcmp( i_arch, "snb" ) == 0) ||
-         (strcmp( i_arch, "hsw" ) == 0) ||
-         (strcmp( i_arch, "knc" ) == 0) ||
-         (strcmp( i_arch, "knl" ) == 0) ||
-         (strcmp( i_arch, "skx" ) == 0))
-    {
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#else\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#pragma message (\"LIBXSMM KERNEL COMPILATION ERROR in: \" __FILE__)\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#error No kernel was compiled, lacking support for current architecture?\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-      l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#endif\n\n" );
-      libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "noarch" ) == 0) ) {
-    } else {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
-      return;
-    }
-  }
-}
-
 void libxsmm_generator_dense_add_flop_counter( libxsmm_generated_code*         io_generated_code,
                                                const libxsmm_gemm_descriptor* i_xgemm_desc ) {
   if ( io_generated_code->code_type == 0 ) {
@@ -538,9 +458,9 @@ void libxsmm_generator_dense_header_kloop( libxsmm_generated_code*             i
                                            const unsigned int                  i_m_blocking,
                                            const unsigned int                  i_k_blocking ) {
   LIBXSMM_UNUSED(i_m_blocking);
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_mov_instruction, i_gp_reg_mapping->gp_reg_kloop, 0);
-  libxsmm_instruction_register_jump_label( io_generated_code, io_loop_label_tracker );
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_kloop, i_k_blocking);
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_mov_instruction, i_gp_reg_mapping->gp_reg_kloop, 0);
+  libxsmm_x86_instruction_register_jump_label( io_generated_code, io_loop_label_tracker );
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_kloop, i_k_blocking);
 }
 
 void libxsmm_generator_dense_footer_kloop( libxsmm_generated_code*             io_generated_code,
@@ -552,10 +472,10 @@ void libxsmm_generator_dense_footer_kloop( libxsmm_generated_code*             i
                                            const unsigned int                  i_max_blocked_k,
                                            const unsigned int                  i_kloop_complete ) {
   LIBXSMM_UNUSED(i_m_blocking);
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_cmp_instruction, i_gp_reg_mapping->gp_reg_kloop, i_max_blocked_k );
-  libxsmm_instruction_jump_back_to_label( io_generated_code, i_micro_kernel_config->alu_jmp_instruction, io_loop_label_tracker );
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_cmp_instruction, i_gp_reg_mapping->gp_reg_kloop, i_max_blocked_k );
+  libxsmm_x86_instruction_jump_back_to_label( io_generated_code, i_micro_kernel_config->alu_jmp_instruction, io_loop_label_tracker );
   if ( i_kloop_complete != 0 ) {
-    libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction,
+    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction,
                                  i_gp_reg_mapping->gp_reg_b, (i_xgemm_desc->k)*(i_micro_kernel_config->datatype_size) );
   }
 }
@@ -565,9 +485,9 @@ void libxsmm_generator_dense_header_nloop( libxsmm_generated_code*             i
                                            const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
                                            const libxsmm_micro_kernel_config*  i_micro_kernel_config,
                                            const unsigned int                  i_n_blocking) {
-  libxsmm_instruction_register_jump_label( io_generated_code, io_loop_label_tracker );
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_nloop, i_n_blocking );
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_mov_instruction, i_gp_reg_mapping->gp_reg_mloop, 0 );
+  libxsmm_x86_instruction_register_jump_label( io_generated_code, io_loop_label_tracker );
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_nloop, i_n_blocking );
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_mov_instruction, i_gp_reg_mapping->gp_reg_mloop, 0 );
 }
 
 
@@ -578,14 +498,14 @@ void libxsmm_generator_dense_footer_nloop( libxsmm_generated_code*             i
                                            const libxsmm_gemm_descriptor*     i_xgemm_desc,
                                            const unsigned int                  i_n_blocking,
                                            const unsigned int                  i_n_done ) {
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_c,
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_c,
                                (i_n_blocking*(i_xgemm_desc->ldc)*(i_micro_kernel_config->datatype_size)) - ((i_xgemm_desc->m)*(i_micro_kernel_config->datatype_size)) );
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
                                i_gp_reg_mapping->gp_reg_b, (i_n_blocking*(i_xgemm_desc->ldb)*(i_micro_kernel_config->datatype_size)) );
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction,
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction,
                                i_gp_reg_mapping->gp_reg_a, ((i_xgemm_desc->m)*(i_micro_kernel_config->datatype_size)) );
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_cmp_instruction, i_gp_reg_mapping->gp_reg_nloop, i_n_done );
-  libxsmm_instruction_jump_back_to_label( io_generated_code, i_micro_kernel_config->alu_jmp_instruction, io_loop_label_tracker );
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_cmp_instruction, i_gp_reg_mapping->gp_reg_nloop, i_n_done );
+  libxsmm_x86_instruction_jump_back_to_label( io_generated_code, i_micro_kernel_config->alu_jmp_instruction, io_loop_label_tracker );
 }
 
 
@@ -594,8 +514,8 @@ void libxsmm_generator_dense_header_mloop( libxsmm_generated_code*             i
                                            const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
                                            const libxsmm_micro_kernel_config*  i_micro_kernel_config,
                                            const unsigned int                  i_m_blocking ) {
-  libxsmm_instruction_register_jump_label( io_generated_code, io_loop_label_tracker );
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_mloop, i_m_blocking );
+  libxsmm_x86_instruction_register_jump_label( io_generated_code, io_loop_label_tracker );
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_mloop, i_m_blocking );
 }
 
 void libxsmm_generator_dense_footer_mloop( libxsmm_generated_code*             io_generated_code,
@@ -607,14 +527,14 @@ void libxsmm_generator_dense_footer_mloop( libxsmm_generated_code*             i
                                            const unsigned int                  i_m_done,
                                            const unsigned int                  i_k_unrolled ) {
   /* advance C pointer */
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
                                i_gp_reg_mapping->gp_reg_c, i_m_blocking*(i_micro_kernel_config->datatype_size) );
   /* B prefetch */
   if ( i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_BL2_VIA_C ||
        i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C ||
        i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD ||
        i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST) {
-    libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
+    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
                                  i_gp_reg_mapping->gp_reg_b_prefetch, i_m_blocking*(i_micro_kernel_config->datatype_size) );
   }
 
@@ -622,28 +542,28 @@ void libxsmm_generator_dense_footer_mloop( libxsmm_generated_code*             i
     /* A prefetch */
     if ( i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2 ||
          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C) {
-      libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction, i_gp_reg_mapping->gp_reg_a_prefetch,
+      libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction, i_gp_reg_mapping->gp_reg_a_prefetch,
                                    ((i_xgemm_desc->k) * (i_micro_kernel_config->datatype_size) * (i_xgemm_desc->lda) ) -
                                      (i_m_blocking * (i_micro_kernel_config->datatype_size)) );
     }
     /* advance A pointer */
-    libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction, i_gp_reg_mapping->gp_reg_a,
+    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction, i_gp_reg_mapping->gp_reg_a,
                                  ((i_xgemm_desc->k) * (i_micro_kernel_config->datatype_size) * (i_xgemm_desc->lda) ) - (i_m_blocking * (i_micro_kernel_config->datatype_size)) );
   } else {
     /* A prefetch */
     if ( i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2 ||
          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C) {
-      libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_a_prefetch,
+      libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_a_prefetch,
                                      (i_m_blocking * (i_micro_kernel_config->datatype_size)) );
     }
     /* advance A pointer */
-    libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_a,
+    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_a,
                                  (i_m_blocking * (i_micro_kernel_config->datatype_size)) );
   }
 
   /* loop handling */
-  libxsmm_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_cmp_instruction, i_gp_reg_mapping->gp_reg_mloop, i_m_done );
-  libxsmm_instruction_jump_back_to_label( io_generated_code, i_micro_kernel_config->alu_jmp_instruction, io_loop_label_tracker );
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_cmp_instruction, i_gp_reg_mapping->gp_reg_mloop, i_m_done );
+  libxsmm_x86_instruction_jump_back_to_label( io_generated_code, i_micro_kernel_config->alu_jmp_instruction, io_loop_label_tracker );
 }
 
 void libxsmm_generator_dense_load_C( libxsmm_generated_code*             io_generated_code,
@@ -690,7 +610,7 @@ void libxsmm_generator_dense_load_C( libxsmm_generated_code*             io_gene
     /* adding to C, so let's load C */
     for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
       for ( l_m = 0; l_m < l_m_blocking; l_m++ ) {
-        libxsmm_instruction_vec_move( io_generated_code,
+        libxsmm_x86_instruction_vec_move( io_generated_code,
                                       i_micro_kernel_config->instruction_set,
                                       i_micro_kernel_config->c_vmove_instruction,
                                       i_gp_reg_mapping->gp_reg_c,
@@ -704,7 +624,7 @@ void libxsmm_generator_dense_load_C( libxsmm_generated_code*             io_gene
     /* overwriting C, so let's xout accumulator */
     for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
       for ( l_m = 0; l_m < l_m_blocking; l_m++ ) {
-        libxsmm_instruction_vec_compute_reg( io_generated_code,
+        libxsmm_x86_instruction_vec_compute_reg( io_generated_code,
                                              i_micro_kernel_config->instruction_set,
                                              i_micro_kernel_config->vxor_instruction,
                                              i_micro_kernel_config->vector_name,
@@ -758,7 +678,7 @@ void libxsmm_generator_dense_store_C( libxsmm_generated_code*             io_gen
   /* adding to C, so let's load C */
   for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
     for ( l_m = 0; l_m < l_m_blocking; l_m++ ) {
-      libxsmm_instruction_vec_move( io_generated_code,
+      libxsmm_x86_instruction_vec_move( io_generated_code,
                                     i_micro_kernel_config->instruction_set,
                                     i_micro_kernel_config->c_vmove_instruction,
                                     i_gp_reg_mapping->gp_reg_c,
@@ -776,7 +696,7 @@ void libxsmm_generator_dense_store_C( libxsmm_generated_code*             io_gen
       unsigned int l_m_advance = 64 / ((i_micro_kernel_config->vector_length) * (i_micro_kernel_config->datatype_size)); /* 64: hardcoded cache line length */
 
       for (l_m = 0; l_m < l_m_blocking; l_m += l_m_advance ) {
-        libxsmm_instruction_prefetch( io_generated_code,
+        libxsmm_x86_instruction_prefetch( io_generated_code,
                                       i_micro_kernel_config->prefetch_instruction,
                                       i_gp_reg_mapping->gp_reg_b_prefetch,
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
