@@ -1,5 +1,5 @@
 !*****************************************************************************!
-!* Copyright (c) 2013-2015, Intel Corporation                                *!
+!* Copyright (c) 2014-2016, Intel Corporation                                *!
 !* All rights reserved.                                                      *!
 !*                                                                           *!
 !* Redistribution and use in source and binary forms, with or without        *!
@@ -68,6 +68,7 @@
         INTEGER(C_INT), PARAMETER :: LIBXSMM_AVG_K = $AVG_K
         INTEGER(C_INT), PARAMETER :: LIBXSMM_FLAGS = $FLAGS
         INTEGER(C_INT), PARAMETER :: LIBXSMM_ILP64 = $ILP64
+        INTEGER(C_INT), PARAMETER :: LIBXSMM_SYNC = $SYNC
         INTEGER(C_INT), PARAMETER :: LIBXSMM_JIT = $JIT
 
         ! Integer type impacting the BLAS interface (LP64: 32-bit, and ILP64: 64-bit).
@@ -312,14 +313,15 @@
           PROCEDURE(LIBXSMM_MMFUNCTION1), POINTER :: fn1
           !DIR$ ATTRIBUTES OFFLOAD:MIC :: ddispatch
           INTERFACE
-            TYPE(C_FUNPTR) PURE FUNCTION ddispatch(                     &
+            PURE FUNCTION ddispatch(                                    &
      &      m, n, k, lda, ldb, ldc, alpha, beta, flags, prefetch)       &
-     &      BIND(C, NAME="libxsmm_dmmdispatch")
+     &      RESULT(fn) BIND(C, NAME="libxsmm_dmmdispatch")
               IMPORT :: C_FUNPTR, C_INT, C_DOUBLE
               INTEGER(C_INT), INTENT(IN), VALUE :: m, n, k
               INTEGER(C_INT), INTENT(IN) :: lda, ldb, ldc
               REAL(C_DOUBLE), INTENT(IN) :: alpha, beta
               INTEGER(C_INT), INTENT(IN) :: flags, prefetch
+              TYPE(C_FUNPTR) :: fn
             END FUNCTION
           END INTERFACE
           IF (.NOT.PRESENT(prefetch).OR.                                &
@@ -380,45 +382,45 @@
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_smmcall_abx
         PURE SUBROUTINE libxsmm_smmcall_abx(fn, a, b, c)
           TYPE(LIBXSMM_SMMFUNCTION), INTENT(IN) :: fn
-          TYPE(C_PTR), INTENT(IN), VALUE :: a, b, c
-          TYPE(C_PTR) :: ab, ba
-          ab = MERGE(a, b, 0.NE.LIBXSMM_COL_MAJOR)
-          ba = MERGE(b, a, 0.NE.LIBXSMM_COL_MAJOR)
-          CALL fn%fn0(ab, ba, c)
+          TYPE(C_PTR), INTENT(IN) :: a, b, c
+          IF (0.NE.LIBXSMM_COL_MAJOR) THEN
+            CALL fn%fn0(a, b, c)
+          ELSE
+            CALL fn%fn0(b, a, c)
+          END IF
         END SUBROUTINE
 
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_dmmcall_abx
         PURE SUBROUTINE libxsmm_dmmcall_abx(fn, a, b, c)
           TYPE(LIBXSMM_DMMFUNCTION), INTENT(IN) :: fn
-          TYPE(C_PTR), INTENT(IN), VALUE :: a, b, c
-          TYPE(C_PTR) :: ab, ba
-          ab = MERGE(a, b, 0.NE.LIBXSMM_COL_MAJOR)
-          ba = MERGE(b, a, 0.NE.LIBXSMM_COL_MAJOR)
-          CALL fn%fn0(ab, ba, c)
+          TYPE(C_PTR), INTENT(IN) :: a, b, c
+          IF (0.NE.LIBXSMM_COL_MAJOR) THEN
+            CALL fn%fn0(a, b, c)
+          ELSE
+            CALL fn%fn0(b, a, c)
+          END IF
         END SUBROUTINE
 
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_smmcall_prx
         PURE SUBROUTINE libxsmm_smmcall_prx(fn, a, b, c, pa, pb, pc)
           TYPE(LIBXSMM_SMMFUNCTION), INTENT(IN) :: fn
-          TYPE(C_PTR), INTENT(IN), VALUE :: a, b, c, pa, pb, pc
-          TYPE(C_PTR) :: ab, ba, pr, pq
-          ab = MERGE( a,  b, 0.NE.LIBXSMM_COL_MAJOR)
-          ba = MERGE( b,  a, 0.NE.LIBXSMM_COL_MAJOR)
-          pr = MERGE(pa, pb, 0.NE.LIBXSMM_COL_MAJOR)
-          pq = MERGE(pb, pa, 0.NE.LIBXSMM_COL_MAJOR)
-          CALL fn%fn1(ab, ba, c, pr, pq, pc)
+          TYPE(C_PTR), INTENT(IN) :: a, b, c, pa, pb, pc
+          IF (0.NE.LIBXSMM_COL_MAJOR) THEN
+            CALL fn%fn1(a, b, c, pa, pb, pc)
+          ELSE
+            CALL fn%fn1(b, a, c, pb, pa, pc)
+          END IF
         END SUBROUTINE
 
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_dmmcall_prx
         PURE SUBROUTINE libxsmm_dmmcall_prx(fn, a, b, c, pa, pb, pc)
           TYPE(LIBXSMM_DMMFUNCTION), INTENT(IN) :: fn
-          TYPE(C_PTR), INTENT(IN), VALUE :: a, b, c, pa, pb, pc
-          TYPE(C_PTR) :: ab, ba, pr, pq
-          ab = MERGE( a,  b, 0.NE.LIBXSMM_COL_MAJOR)
-          ba = MERGE( b,  a, 0.NE.LIBXSMM_COL_MAJOR)
-          pr = MERGE(pa, pb, 0.NE.LIBXSMM_COL_MAJOR)
-          pq = MERGE(pb, pa, 0.NE.LIBXSMM_COL_MAJOR)
-          CALL fn%fn1(ab, ba, c, pr, pq, pc)
+          TYPE(C_PTR), INTENT(IN) :: a, b, c, pa, pb, pc
+          IF (0.NE.LIBXSMM_COL_MAJOR) THEN
+            CALL fn%fn1(a, b, c, pa, pb, pc)
+          ELSE
+            CALL fn%fn1(b, a, c, pb, pa, pc)
+          END IF
         END SUBROUTINE
 
         !DIR$ ATTRIBUTES OFFLOAD:MIC :: libxsmm_smmcall_abc
