@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2013-2015, Intel Corporation                                **
+** Copyright (c) 2014-2016, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -60,8 +60,9 @@
 #define LIBXSMM_ILP64 $ILP64
 #define LIBXSMM_ALPHA $ALPHA
 #define LIBXSMM_BETA $BETA
+#define LIBXSMM_SYNC $SYNC
 #define LIBXSMM_JIT $JIT
-
+$LIBXSMM_OFFLOAD_BUILD
 #include "libxsmm_frontend.h"
 
 
@@ -93,19 +94,55 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE libxsmm_dmmfunction libxsmm_dmmdispatch(in
   const double* alpha, const double* beta,
   const int* flags, const int* prefetch);
 
-/** Dispatched general dense matrix multiplication (single-precision). */
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_sgemm(const char* transa, const char* transb,
+/** Dispatched general dense matrix multiplication (single-precision); can be called from F77 code. */
+LIBXSMM_INLINE_EXPORT LIBXSMM_RETARGETABLE void libxsmm_sgemm(const char* transa, const char* transb,
   const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
   const float* alpha, const float* a, const libxsmm_blasint* lda,
   const float* b, const libxsmm_blasint* ldb,
-  const float* beta, float* c, const libxsmm_blasint* ldc);
+  const float* beta, float* c, const libxsmm_blasint* ldc)
+#if defined(LIBXSMM_BUILD)
+;
+#else
+{ int flags = (0 != transa
+    ? (('N' == *transa || 'n' == *transa) ? (LIBXSMM_FLAGS & ~LIBXSMM_GEMM_FLAG_TRANS_A)
+                                          : (LIBXSMM_FLAGS |  LIBXSMM_GEMM_FLAG_TRANS_A))
+    : LIBXSMM_FLAGS);
+  flags = (0 != transb
+    ? (('N' == *transb || 'n' == *transb) ? (flags & ~LIBXSMM_GEMM_FLAG_TRANS_B)
+                                          : (flags |  LIBXSMM_GEMM_FLAG_TRANS_B))
+    : flags);
+  LIBXSMM_SGEMM(flags, *m, *n, *k,
+    0 != alpha ? *alpha : ((float)LIBXSMM_ALPHA),
+    a, *(lda ? lda : LIBXSMM_LD(m, k)), b, *(ldb ? ldb : LIBXSMM_LD(k, n)),
+    0 != beta ? *beta : ((float)LIBXSMM_BETA),
+    c, *(ldc ? ldc : LIBXSMM_LD(m, n)));
+}
+#endif
 
-/** Dispatched general dense matrix multiplication (double-precision). */
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_dgemm(const char* transa, const char* transb,
+/** Dispatched general dense matrix multiplication (double-precision); can be called from F77 code. */
+LIBXSMM_INLINE_EXPORT LIBXSMM_RETARGETABLE void libxsmm_dgemm(const char* transa, const char* transb,
   const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
   const double* alpha, const double* a, const libxsmm_blasint* lda,
   const double* b, const libxsmm_blasint* ldb,
-  const double* beta, double* c, const libxsmm_blasint* ldc);
+  const double* beta, double* c, const libxsmm_blasint* ldc)
+#if defined(LIBXSMM_BUILD)
+;
+#else
+{ int flags = (0 != transa
+    ? (('N' == *transa || 'n' == *transa) ? (LIBXSMM_FLAGS & ~LIBXSMM_GEMM_FLAG_TRANS_A)
+                                          : (LIBXSMM_FLAGS |  LIBXSMM_GEMM_FLAG_TRANS_A))
+    : LIBXSMM_FLAGS);
+  flags = (0 != transb
+    ? (('N' == *transb || 'n' == *transb) ? (flags & ~LIBXSMM_GEMM_FLAG_TRANS_B)
+                                          : (flags |  LIBXSMM_GEMM_FLAG_TRANS_B))
+    : flags);
+  LIBXSMM_DGEMM(flags, *m, *n, *k,
+    0 != alpha ? *alpha : ((double)LIBXSMM_ALPHA),
+    a, *(lda ? lda : LIBXSMM_LD(m, k)), b, *(ldb ? ldb : LIBXSMM_LD(k, n)),
+    0 != beta ? *beta : ((double)LIBXSMM_BETA),
+    c, *(ldc ? ldc : LIBXSMM_LD(m, n)));
+}
+#endif
 
 /** General dense matrix multiplication based on LAPACK/BLAS (single-precision). */
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_blas_sgemm(const char* transa, const char* transb,
@@ -214,7 +251,7 @@ public:
 };
 
 /** Dispatched general dense matrix multiplication (single-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_gemm(const char* transa, const char* transb, const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
+inline LIBXSMM_RETARGETABLE void libxsmm_gemm(const char* transa, const char* transb, const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
   const float* alpha, const float* a, const libxsmm_blasint* lda, const float* b, const libxsmm_blasint* ldb,
   const float* beta, float* c, const libxsmm_blasint* ldc)
 {
@@ -222,7 +259,7 @@ LIBXSMM_RETARGETABLE inline void libxsmm_gemm(const char* transa, const char* tr
 }
 
 /** Dispatched general dense matrix multiplication (double-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_gemm(const char* transa, const char* transb, const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
+inline LIBXSMM_RETARGETABLE void libxsmm_gemm(const char* transa, const char* transb, const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
   const double* alpha, const double* a, const libxsmm_blasint* lda, const double* b, const libxsmm_blasint* ldb,
   const double* beta, double* c, const libxsmm_blasint* ldc)
 {
@@ -230,7 +267,7 @@ LIBXSMM_RETARGETABLE inline void libxsmm_gemm(const char* transa, const char* tr
 }
 
 /** Dispatched general dense matrix multiplication (single-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_sgemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+inline LIBXSMM_RETARGETABLE void libxsmm_sgemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   const float* alpha, const float* a, const libxsmm_blasint* lda, const float* b, const libxsmm_blasint* ldb,
   const float* beta, float* c, const libxsmm_blasint* ldc)
 {
@@ -238,7 +275,7 @@ LIBXSMM_RETARGETABLE inline void libxsmm_sgemm(const char* transa, const char* t
 }
 
 /** Dispatched general dense matrix multiplication (double-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_dgemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+inline LIBXSMM_RETARGETABLE void libxsmm_dgemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   const double* alpha, const double* a, const libxsmm_blasint* lda, const double* b, const libxsmm_blasint* ldb,
   const double* beta, double* c, const libxsmm_blasint* ldc)
 {
@@ -246,7 +283,7 @@ LIBXSMM_RETARGETABLE inline void libxsmm_dgemm(const char* transa, const char* t
 }
 
 /** Dispatched general dense matrix multiplication (single-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_gemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+inline LIBXSMM_RETARGETABLE void libxsmm_gemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   const float* alpha, const float* a, const libxsmm_blasint* lda, const float* b, const libxsmm_blasint* ldb,
   const float* beta, float* c, const libxsmm_blasint* ldc)
 {
@@ -254,7 +291,7 @@ LIBXSMM_RETARGETABLE inline void libxsmm_gemm(const char* transa, const char* tr
 }
 
 /** Dispatched general dense matrix multiplication (double-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_gemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+inline LIBXSMM_RETARGETABLE void libxsmm_gemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   const double* alpha, const double* a, const libxsmm_blasint* lda, const double* b, const libxsmm_blasint* ldb,
   const double* beta, double* c, const libxsmm_blasint* ldc)
 {
@@ -262,7 +299,7 @@ LIBXSMM_RETARGETABLE inline void libxsmm_gemm(const char* transa, const char* tr
 }
 
 /** General dense matrix multiplication based on LAPACK/BLAS (single-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_blas_gemm(const char* transa, const char* transb, const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
+inline LIBXSMM_RETARGETABLE void libxsmm_blas_gemm(const char* transa, const char* transb, const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
   const float* alpha, const float* a, const libxsmm_blasint* lda, const float* b, const libxsmm_blasint* ldb,
   const float* beta, float* c, const libxsmm_blasint* ldc)
 {
@@ -270,7 +307,7 @@ LIBXSMM_RETARGETABLE inline void libxsmm_blas_gemm(const char* transa, const cha
 }
 
 /** General dense matrix multiplication based on LAPACK/BLAS (double-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_blas_gemm(const char* transa, const char* transb, const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
+inline LIBXSMM_RETARGETABLE void libxsmm_blas_gemm(const char* transa, const char* transb, const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* k,
   const double* alpha, const double* a, const libxsmm_blasint* lda, const double* b, const libxsmm_blasint* ldb,
   const double* beta, double* c, const libxsmm_blasint* ldc)
 {
@@ -278,7 +315,7 @@ LIBXSMM_RETARGETABLE inline void libxsmm_blas_gemm(const char* transa, const cha
 }
 
 /** General dense matrix multiplication based on LAPACK/BLAS (single-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_blas_sgemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+inline LIBXSMM_RETARGETABLE void libxsmm_blas_sgemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   const float* alpha, const float* a, const libxsmm_blasint* lda, const float* b, const libxsmm_blasint* ldb,
   const float* beta, float* c, const libxsmm_blasint* ldc)
 {
@@ -286,7 +323,7 @@ LIBXSMM_RETARGETABLE inline void libxsmm_blas_sgemm(const char* transa, const ch
 }
 
 /** General dense matrix multiplication based on LAPACK/BLAS (double-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_blas_dgemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+inline LIBXSMM_RETARGETABLE void libxsmm_blas_dgemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   const double* alpha, const double* a, const libxsmm_blasint* lda, const double* b, const libxsmm_blasint* ldb,
   const double* beta, double* c, const libxsmm_blasint* ldc)
 {
@@ -294,7 +331,7 @@ LIBXSMM_RETARGETABLE inline void libxsmm_blas_dgemm(const char* transa, const ch
 }
 
 /** General dense matrix multiplication based on LAPACK/BLAS (single-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_blas_gemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+inline LIBXSMM_RETARGETABLE void libxsmm_blas_gemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   const float* alpha, const float* a, const libxsmm_blasint* lda, const float* b, const libxsmm_blasint* ldb,
   const float* beta, float* c, const libxsmm_blasint* ldc)
 {
@@ -302,7 +339,7 @@ LIBXSMM_RETARGETABLE inline void libxsmm_blas_gemm(const char* transa, const cha
 }
 
 /** General dense matrix multiplication based on LAPACK/BLAS (double-precision). */
-LIBXSMM_RETARGETABLE inline void libxsmm_blas_gemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+inline LIBXSMM_RETARGETABLE void libxsmm_blas_gemm(const char* transa, const char* transb, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   const double* alpha, const double* a, const libxsmm_blasint* lda, const double* b, const libxsmm_blasint* ldb,
   const double* beta, double* c, const libxsmm_blasint* ldc)
 {
